package bitel.billing.test.convert;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import bitel.billing.common.CommonTimeUtils;
import bitel.billing.common.CommonUtils;
import bitel.billing.server.ApplicationModule;
import bitel.billing.server.contract.bean.ContractMemo;
import bitel.billing.server.contract.bean.ContractMemoManager;
import bitel.billing.server.contract.bean.ContractUtils;
import bitel.billing.server.tariff.TariffTreeBuilder;
import bitel.billing.server.util.DefaultServerSetup;
import bitel.billing.server.util.SetupData;
import bitel.billing.server.util.Utils;
import bitel.billing.server.util.ZipUtils;

public class ConvertUtil
{
    /**
     * 
     * @param group    !
     * @param setup 
     */
	public static void removeContracts( long group, DefaultServerSetup setup )
	{
		Connection con = null;
		Connection conRem = null;
		try
		{
			con = setup.getDBConnection();
			
			conRem= setup.getDBConnection();
			ContractRemover remover = new ContractRemover( setup, conRem, null );
			
			
			String query = "SELECT	id FROM  contract WHERE gr & ? != 0 ";
			PreparedStatement ps  = con.prepareStatement( query );
			ps.setLong( 1, group );
			ResultSet rs = ps.executeQuery();
						
			
			int count = 0;
			while ( rs.next() )
			{
				int cid = rs.getInt( 1 );
				remover.deleteContract( cid );
				count ++;
				
				if ( count > 1 && count % 2000 == 0)
				{
					//System.gc();
					closeConnection( conRem );
					conRem= setup.getDBConnection();
					remover = new ContractRemover( setup, conRem, null );
					
				}
			}
			rs.close();
			ps.close();
			
			System.out.println( count + " contracts was deleted..");
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			//setup.returnConToPool( con );
			closeConnection( con );
			closeConnection( conRem );
		}
		
		
	}

	public static void closeConnection( Connection con )
	{
		try
		{
			con.close();
		} catch (SQLException e)
		{			
		}
	}		
	
	public static Map<String, Integer> getBillingTariffMap( DefaultServerSetup setup )
    {
        Map<String, Integer> tarifMap = new HashMap<String, Integer>();
        Connection con = null;
        try
        {
            con = setup.getDBConnectionFromPool();
            String query = "SELECT id, title FROM tariff_plan";
            PreparedStatement ps = con.prepareStatement( query );
            ResultSet rs = ps.executeQuery();
            while ( rs.next() )
            {
                tarifMap.put( rs.getString( 2 ), rs.getInt( 1 ) );
            }
            rs.close();
            ps.close();
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        finally
        {
            setup.returnConToPool( con );
        }
        return tarifMap;
    }
	
	public  static Map<String, Integer >  getBillingStreetMap( DefaultServerSetup setup )
	{
		Map<String, Integer > tarifMap = new HashMap<String, Integer>();
		Connection con = null;
		try
		{
			con = setup.getDBConnectionFromPool();
			
			String query = "SELECT id, title FROM address_street";
			
			 
			
			PreparedStatement ps =  con.prepareStatement(   query );            
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				tarifMap.put( rs.getString( 2 ), rs.getInt( 1 )  );
				
			}
			rs.close();
			ps.close();
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			setup.returnConToPool( con );
		}
		
		
		return tarifMap; 
	}		
	
	/**
     *   - .
     * @param cid
     * @param con
     * @param value ,  null/ -  ,   . 
     * @param pid ,   - .
     * @throws SQLException
     */
	public static void addContactParamString(int cid, Connection con, String value, int pid ) throws SQLException
	{
	    //try{
		if ( pid>0 )
		{
			String query = "DELETE FROM contract_parameter_type_1 WHERE cid=? AND pid=?";
			PreparedStatement ps =  con.prepareStatement( query );
			ps.setInt( 1, cid );
			ps.setInt( 2, pid );
			ps.executeUpdate();
			ps.close();
			if ( !Utils.isEmptyString( value ) )
			{
				query = "INSERT INTO contract_parameter_type_1 " +
						"( cid, pid, val ) VALUES ( ?, ?, ? )";
				ps = con.prepareStatement( query );
				ps.setInt( 1, cid );
				ps.setInt( 2, pid );
				ps.setString( 3, value );
				ps.executeUpdate();
				ps.close();
			}			
		}	
	    //}
		//catch (Exception e) {
        //    e.printStackTrace();
        //    System.out.println( "value='"+value+"'" );
        //    System.exit( 1 );
        //}
	}	

	/**
	 *   - .
	 * @param cid
	 * @param con
	 * @param date ,  null -  ,   . 
	 * @param datePid  ,   - .
	 * @throws SQLException
	 */
    public static void addContactParamDate( int cid, Connection con, Date date, int datePid ) throws SQLException
    {
        if( datePid > 0 )
        {
            String query = "DELETE FROM contract_parameter_type_6 WHERE cid=? AND pid=?";
            PreparedStatement ps =  con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, datePid );
            ps.executeUpdate();
            ps.close();
            if ( date != null )
            {
                query = "INSERT INTO contract_parameter_type_6 ( cid, pid, val ) VALUES ( ?, ?, ? )";
                ps = con.prepareStatement( query );
                ps.setInt( 1, cid );
                ps.setInt( 2, datePid );
                ps.setDate( 3, CommonTimeUtils.convertDateToSQL( date ) );
                ps.executeUpdate();
                ps.close();
            }
        }
    }
	
    /**
     *   - .
     * @param cid
     * @param con
     * @param email ,  null -  ,   . 
     * @param emailPid ,   - .
     * @throws SQLException
     */
    public static void addContactParamEmail( int cid,
                                             Connection con,
                                             String email,
                                             int emailPid ) throws SQLException
    {
        if( emailPid > 0 )
        {            
            String query = "DELETE FROM contract_parameter_type_3 WHERE cid=? AND pid=?";
            PreparedStatement ps =  con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, emailPid );
            ps.executeUpdate();
            ps.close();
            if ( email != null )
            {
                query = "INSERT INTO contract_parameter_type_3 ( cid, pid, email ) VALUES ( ?, ?, ? )";
                ps = con.prepareStatement( query );
                ps.setInt( 1, cid );
                ps.setInt( 2, emailPid );
                ps.setString( 3, email );
                ps.executeUpdate();
                ps.close();
            }
        }
    }
    
    public static void addContactParamFlag( int cid, Connection con, boolean flag, int flagPid ) throws SQLException
    {
        if ( flagPid > 0 )
        {
            String query = "DELETE FROM contract_parameter_type_5 WHERE cid=? AND pid=?";
            PreparedStatement ps = con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, flagPid );
            ps.executeUpdate();
            ps.close();
            //
            query = "INSERT INTO contract_parameter_type_5 ( cid, pid, val ) VALUES ( ?, ?, ? )";
            ps = con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, flagPid );
            ps.setBoolean( 3, flag );
            ps.executeUpdate();
            ps.close();
        }
    }
	
	public static int addNewTariff( Connection con, String title ) throws SQLException
	{
		String query;
		PreparedStatement ps;
		//   
		query = "INSERT INTO tariff_plan ( title ) values ( ? )";
		ps = con.prepareStatement( query );
		ps.setString( 1,  title); 
		ps.executeUpdate();

		int tpId = lastInsertId( con );
		int treeId = new TariffTreeBuilder( con ).createTree();
		
		ps.close();
		
		query = "INSERT INTO tariff_tree_link (tpid, tree_id) VALUES (?, ?)";
		ps = con.prepareStatement( query );
		
		ps.setInt( 1, tpId );
		ps.setInt( 2, treeId );
		
		ps.executeUpdate();
		ps.close();
		return tpId;
	}	

	 public static int lastInsertId( Connection con ) throws SQLException
	 {
	 	int id = -1;
	 	
	 	PreparedStatement ps =  con.prepareStatement(   "SELECT last_insert_id() as id" );
	 	
	    ResultSet rs = ps.executeQuery( "SELECT last_insert_id() as id" );
	    while( rs.next() )
	    {
	    	id = rs.getInt( 1 );
	    }
	    rs.close();
	    ps.close();
	    return id;
	}		
	 
	//    4.5 ..    4.4       
	 //-  ActionBase ..        
	 
	/*protected String getAddress()
    {
        return getAddress( getParameter( "index", "" ), getParameter( "cityStr", "" ), getParameter( "area", "" ),
                getParameter( "quarter", "" ), getParameter( "streetStr", "" ), getParameter( "house", "" ), getParameter(
                        "frac", "" ), getParameter( "flat", "" ), getParameter( "room", "" ), getParameter( "pod", "" ),
                getParameter( "floor", "" ), getParameter( "comment" ) );
    }	 
	 */
	public static final String getAddress( String index, String city, String area, String quarter, String street, String house,
	                                    String frac, String flat, String room, String pod, String floor, String comment,
	                                     String addressFormat)
    {
        //  
            
        return getAddress( addressFormat, false, index, city, area, quarter, street, house, frac, flat, room, pod, floor,
                comment );
 
    }	
	
	public static final String getAddress( final String _address, final boolean old, final String index, final String city,
	                                    String area, String quarter, final String street, String house, final String frac, final String flat,
	                                    final String room, final String pod, final String floor, final String comment )
    {
        String address = _address;

        //  
        if( old )
        {
            if( area == null )
            {
                area = " ";
            }
            if( quarter == null )
            {
                quarter = " ";
            }

            address = address.replaceAll( "\\$\\{index\\}", Utils.maskNull( index ) );
            address = address.replaceAll( "\\$\\{city\\}", Utils.maskNull( city ) );
            address = address.replaceAll( "\\$\\{area\\}", Utils.maskNull( area ) );
            address = address.replaceAll( "\\$\\{quarter\\}", Utils.maskNull( quarter ) );
            address = address.replaceAll( "\\$\\{street\\}", Utils.maskNull( street ) );
            address = address.replaceAll( "\\$\\{house\\}", Utils.maskNull( house ) );
            address = address.replaceAll( "\\$\\{frac\\}", Utils.maskNull( frac ) );
            address = address.replaceAll( "\\$\\{flat\\}", Utils.maskNull( flat ) );
            address = address.replaceAll( "\\$\\{room\\}", Utils.maskNull( room ) );
            address = address.replaceAll( "\\$\\{pod\\}", Utils.maskNull( pod ) );
            address = address.replaceAll( "\\$\\{floor\\}", Utils.maskNull( floor ) );
        }
        //  
        else
        {
            if( " ".equals( area ) || " ".equals( area ) )
            {
                area = null;
            }
            if( " ".equals( quarter ) || " ".equals( quarter ) )
            {
                quarter = null;
            }
            if( "0".equals( house ) )
            {
                house = null;
            }

            address = insertAddressPart( address, "index", index );
            address = insertAddressPart( address, "city", city );
            address = insertAddressPart( address, "area", area );
            address = insertAddressPart( address, "quarter", quarter );
            address = insertAddressPart( address, "street", street );
            address = insertAddressPart( address, "house", house );
            address = insertAddressPart( address, "frac", frac );
            address = insertAddressPart( address, "flat", flat );
            address = insertAddressPart( address, "room", room );
            address = insertAddressPart( address, "pod", pod );
            address = insertAddressPart( address, "floor", floor );
            address = insertAddressPart( address, "comment", comment );
        }

        return address;
    }	
	protected static HashMap<String, Pattern> insertAddressPartPatterns = new HashMap<String, Pattern>( 12 ); 
	private static String insertAddressPart( String address, String key, String value )
    {
        StringBuilder result = new StringBuilder( address.length() );

        Pattern p = insertAddressPartPatterns.get( key );
        if( p == null )
        {
            p = Pattern.compile( "\\(([\\w--\\,\\.\\s\\[\\]\\\\/#]*)\\$\\{" + key + "\\}([\\w--\\,\\.\\s\\[\\]\\\\/#]*)\\)" );
            insertAddressPartPatterns.put( key, p );
        }

        Matcher m = p.matcher( address );
        if( m.find() )
        {
            if( !CommonUtils.isBlankString( value ) )
            {
                result.append( address.substring( 0, m.start() ) );

                String prefix = m.group( 1 );

                if( prefix.startsWith( "," ) && result.length() == 0 )
                {
                    prefix = prefix.substring( 1 );
                }

                result.append( prefix );
                result.append( value );
                result.append( m.group( 2 ) );
                result.append( address.substring( m.end() ) );
            }
            else
            {
                result.append( address.substring( 0, m.start() ) );
                result.append( address.substring( m.end() ) );
            }
        }
        else
        {
            result.append( address );
        }

        return result.toString();
    }	 
	
	
	
	//server/util/Utils -  rs.close()      ..  !!  
	
	public static List<String> deleteRowsFromTable( Connection con, String table, String keyColumn, String keyValue,
            String retColumn ) throws SQLException
    {
        List<String> result = new ArrayList<String>();
        if( Utils.tableExists( con, table ) )
        {
            String query = "SELECT * FROM " + table + " WHERE " + keyColumn + "='" + keyValue + "'";
            ResultSet rs = con.createStatement().executeQuery( query );

            ResultSetMetaData md = rs.getMetaData();
            int columns = md.getColumnCount();

            while( rs.next() )
            {
                for( int i = 1; i <= columns; i++ )
                {
                    String name = md.getColumnName( i );
                    if( retColumn != null && name.equals( retColumn ) )
                        result.add( rs.getString( i ) );
                }
            }
			rs.close();
            query = "DELETE FROM " + table + " WHERE " + keyColumn + "='" + keyValue + "'";
            con.createStatement().executeUpdate( query );
        }

        return result;
    }	
	
	/**
	 *     
	 * @param filePath    
	 * @param charsetName  
	 * @param splitRegexp     - 
	 * @param fields     (   )
	 * @return  ( =>)
	 * @throws UnsupportedEncodingException
	 * @throws IOException
	 */
	public static List<Map<String, String>> loadTxtDB( String filePath,
                                                       String charsetName,
                                                       String splitRegexp,
                                                       String... fields )
        throws UnsupportedEncodingException, IOException
    {
        List<Map<String, String>> result = new ArrayList<Map<String, String>>();
        InputStream stream = new FileInputStream( filePath );
        //
        int count = 0;
        byte buff[] = new byte[1024];
        StringBuffer strBuffer = new StringBuffer();
        //
        while ( (count = stream.read( buff, 0, 1024 )) > 0 )
        {
            strBuffer.append( new String( buff, 0, count, charsetName ) );
        }
        String lines[] = strBuffer.toString().split( "\n" );
        for ( String line : lines )
        {
            Map<String, String> map = new HashMap<String, String>();
            String words[] = line.split( splitRegexp );
            int idx = 0;
            for ( String fieldName : fields )
            {
                map.put( fieldName, words[idx++].trim() );
            }
            result.add( map );
        }
        return result;
    }
	
	/**
	 *    .   TelnetTV,    . 
	 * @param setup
	 * @return    (_=>(_[/_]=>_)
	 */
	public static Map<Integer, Map<String, Integer>> getBillingHouseMap( DefaultServerSetup setup )
    {
        Map<Integer, Map<String, Integer>> result = new HashMap<Integer, Map<String, Integer>>();
        Connection con = null;
        try
        {
            con = setup.getDBConnectionFromPool();
            String query = "SELECT streetid, house, id, frac FROM address_house";
            PreparedStatement ps = con.prepareStatement( query );
            ResultSet rs = ps.executeQuery();
            while ( rs.next() )
            {
                int steetId = rs.getInt( 1 );
                int house = rs.getInt( 2 );
                int houseId = rs.getInt( 3 );
                String frac = rs.getString( 4 );
                Map<String, Integer> houses = result.get( steetId );
                if ( houses == null )
                {
                    houses = new HashMap<String, Integer>();
                    result.put( steetId, houses );
                }
                houses.put( String.valueOf( house ) + ((frac!=null && !frac.isEmpty())?("/" + String.valueOf( frac )):("")) , houseId );
            }
            rs.close();
            ps.close();
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        finally
        {
            setup.returnConToPool( con );
        }
        return result;
    } 
	
	public static Integer parseInteger( String str, int devaultValue )
	{
	    Integer ret = null;
	    try{
	        ret = Integer.valueOf( str );
	    }
	    catch (Throwable e)
	    {
	        ret = devaultValue;
        }
	    return ret;
	}
		
    /**
     *    .
     * @param cid
     * @param groupBitmask
     * @param con
     * @throws SQLException
     */
	public static void addContactGroup( int cid, Long groupBitmask, Connection con ) throws SQLException
    {
        if( groupBitmask != null )
        {
            String query = "UPDATE contract SET gr=gr|? WHERE id=?";
            PreparedStatement ps = con.prepareStatement( query );
            ps.setLong( 1, groupBitmask );
            ps.setInt( 2, cid );
            ps.executeUpdate();
            ps.close();
        }
    }
	
	public static void updateContactComment( int cid, String comment, Connection con ) throws SQLException, UnsupportedEncodingException
    {
        String query = "UPDATE contract SET comment=? WHERE id=?";
        PreparedStatement ps = con.prepareStatement( query );
        ps.setString( 1, comment );
        ps.setInt( 2, cid );
        ps.executeUpdate();
        ps.close();
    }
	
	public static void updateContactClosesumma( int cid, double closesumma, Connection con )
        throws SQLException, UnsupportedEncodingException
    {
        String query = "UPDATE contract SET closesumma=? WHERE id=?";
        PreparedStatement ps = con.prepareStatement( query );
        ps.setDouble( 1, closesumma );
        ps.setInt( 2, cid );
        ps.executeUpdate();
        ps.close();
    }
	
	/**
	 *    - 
	 * @param number
	 * @param nullCount
	 * @return
	 */
    static String addNull( int number, int nullCount )
    {
        StringBuilder sb = new StringBuilder(nullCount);
        String nums = String.valueOf( number );
        for( int i=0; i<nullCount-nums.length(); i++ )
        {
            sb.append( "0" );
        }
        sb.append( nums );
        return sb.toString();
    }
    
    static void updateComments( Connection bgCon, int cid, String comments ) throws UnsupportedEncodingException, SQLException
    {        
        if( CommonUtils.notBlankString( comments ) )
        {
            ContractMemoManager memoManager = new ContractMemoManager( bgCon );
            ContractMemo memo = new ContractMemo();
            memo.setUserId( 0 );
            memo.setContractId( cid );
            memo.setComment( comments );
            memo.setSubject( "" );
            memoManager.updateMemo( memo );
        }
    }
}

// TODO     , .   4.4 Contract Remover 
//  c SetupData ..
//   4.5  


class ContractRemover
{
	private DefaultServerSetup setup;
	private Connection con;

	private Map<Integer, String> modulesPackages = new HashMap<Integer, String>();
	private Map<String, Integer> serviceModules = new HashMap<String, Integer>();
	private Map<String,PreparedStatement> preparedStatments = new HashMap<String,PreparedStatement>();

    private ContractUtils _cu;

	private File archiveFolder;
	private File currentFolder;


	public ContractRemover( DefaultServerSetup setup, Connection con, String folder )
	{
		this.setup = setup;
		this.con = con;
		initServerPackages();

        _cu = new ContractUtils( con );

		initFolders( folder );
	}

	private void initFolders( String folder )
	{
		archiveFolder = new File( "archive" );

		if( archiveFolder.exists() && archiveFolder.isDirectory() )
		{
			StringBuffer currentFolderPath = new StringBuffer( archiveFolder.getAbsolutePath() + "/" );
			if ( folder != null && !folder.equals( "/" ) && folder.length() > 0 )
			{
				currentFolderPath.append( folder );
				currentFolderPath.append( "/" );
			}

			currentFolder = new File( currentFolderPath.toString() );
			if( !currentFolder.exists() || !currentFolder.isDirectory() )
			{
				if( !currentFolder.mkdir() ) currentFolder = null;
			}
		}
		else
		{
			archiveFolder = null;
		}
	}

	private void initServerPackages()
	{
		try
		{
			String query = "SELECT id, package_server, title FROM module";
			PreparedStatement ps = getPS( query );
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				Integer key = new Integer( rs.getInt( 1 ) );
				modulesPackages.put( key, rs.getString( 2 ) );
			}

			query = "SELECT id, mid FROM service";
			rs = getPS( query ).executeQuery();
			while ( rs.next() )
			{
				serviceModules.put( rs.getString( 1 ), new Integer( rs.getInt( 2 ) ) );
			}
		}
		catch ( Exception ex )
		{
			ex.printStackTrace();
		}
	}

	public String removeContracts( List<Integer> cids,
                                   boolean save )
    {
        StringBuffer result = new StringBuffer();
        for ( Integer cid : cids )
        {
            boolean delete = true;
            if ( save )
            {
                delete = saveContract( cid.intValue() );
            }
            if ( delete )
            {
                String msg = deleteContract( cid.intValue() );
                if ( msg != null && msg.length() > 0 )
                {
                    result.append( "  id=" + cid.intValue() + "  !\n" );
                }
            }
        }
        return result.toString();
    }

	private PreparedStatement getPS( String query )
	{
		PreparedStatement ps = preparedStatments.get( query );
		if ( ps == null )
		{
			try
			{
				ps = con.prepareStatement( query );
				preparedStatments.put( query, ps );
			}
			catch ( Exception ex )
			{
				ex.printStackTrace();
			}
		}
		return ps;
	}
	
	/**   ps  .  , ! */
	private void freedom4ps()
    {
        try
        {
            for ( PreparedStatement ps : preparedStatments.values() )
            {
                ps.close();
            }
        }
        catch( SQLException e )
        {
            e.printStackTrace();
        }
    }

	public String deleteContract( int cid )
	{
		String query = null;
		PreparedStatement ps = null;
		try
		{
            Set<Integer> usingModules = new HashSet<Integer>();
			executeDelete( "DELETE FROM contract WHERE id=?", cid );
			query = "SELECT t2.mid FROM contract_service AS t1, " +
					"service AS t2 WHERE t1.sid=t2.id AND t1.cid=?";
			ps = getPS( query );
			ps.setInt( 1, cid );
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				usingModules.add( new Integer( rs.getInt( 1 ) ) );
			}
			rs.close();

			for ( Integer mid : usingModules )
			{
				ApplicationModule am = getModule( modulesPackages.get( mid ) );
				if ( am != null )
				{
					am.deleteContract( cid, mid.intValue() );
				}
			}

			executeDelete( "DELETE FROM contract_service WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_payment WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_charge WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_tariff WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_balance WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_account WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_1 WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_2 WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_3 WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_4 WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_5 WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_6 WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_parameter_type_7 WHERE cid=?", cid );
			/*executeDelete( "DELETE FROM contract_status WHERE cid=?", cid );
			executeDelete( "DELETE FROM contract_status_log WHERE cid=?", cid );*/
			executeDelete( "DELETE FROM user_dispatch WHERE cid=?", cid );
			/*executeDelete( "DELETE FROM register_task WHERE cid=?", cid );
			executeDelete( "DELETE FROM register_call WHERE cid=?", cid );*/
            
            String cidStr = String.valueOf( cid );
            
             //    
            List<String> objects = ConvertUtil.deleteRowsFromTable( con,  "object", "cid", cidStr, "id" );
            for( String objectId : objects )
            {
            	ConvertUtil.deleteRowsFromTable( con, "object_param_value_address", "object_id", objectId, null );
            	ConvertUtil.deleteRowsFromTable( con, "object_param_value_date", "object_id", objectId, null );
            	ConvertUtil.deleteRowsFromTable( con, "object_param_value_list", "object_id", objectId, null );
            	ConvertUtil.deleteRowsFromTable( con, "object_param_value_text", "object_id", objectId, null );
            }
		}
		catch ( Exception ex )
		{
			ex.printStackTrace();
			return ex.getMessage();
		}
		return null;
	}

	private void executeDelete( String query, int cid )
		throws SQLException
	{
		PreparedStatement ps = getPS( query );
		ps.setInt( 1, cid );
		ps.executeUpdate();
	}

	private boolean serializeContract( Document doc, int cid )
	{
		boolean result = false;

		try
		{
			//  
			String cidStr = String.valueOf( cid );
			Element tables = doc.createElement( "tables" );
			doc.appendChild( tables );

			Utils.addItemsFromRS( con, tables, "contract", "id", cidStr, null, false, null, null );
			Utils.addItemsFromRS( con, tables, "contract_account", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_balance", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_payment", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_charge", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_1", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_2", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_3", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_4", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_5", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_6", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_parameter_type_7", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "user_dispatch", "cid", cidStr, null, false, null, null  );
			Utils.addItemsFromRS( con, tables, "contract_tariff", "cid", cidStr, null, false, null, null  );
			
            //    
            List<String> objects = Utils.addItemsFromRS( con, tables, "object", "cid", cidStr, "id", false, null, null );
            for( String objectId : objects )
            {
                Utils.addItemsFromRS( con, tables, "object_param_value_address", "object_id", objectId, null, false, null, null );
                Utils.addItemsFromRS( con, tables, "object_param_value_date", "object_id", objectId, null, false, null, null );
                Utils.addItemsFromRS( con, tables, "object_param_value_list", "object_id", objectId, null, false, null, null );
                Utils.addItemsFromRS( con, tables, "object_param_value_text", "object_id", objectId, null, false, null, null );
            }
            
            Set<Integer> savedMids = new HashSet<Integer>();
            
            //    
            List<String> sids = Utils.addItemsFromRS( con, tables, "contract_service", "cid", cidStr, "sid", false, null, null  );
			for ( String sid : sids )
			{
				Integer mid = serviceModules.get( sid );
				if ( mid != null && !savedMids.contains( mid ) )
				{
					String packageServer = modulesPackages.get( mid );
                    if( packageServer != null )
                    {
                        ApplicationModule am = getModule( packageServer );
                        if ( am != null )am.serializeContract( cid, mid.intValue(),
                                                               tables, false );
                    }
					savedMids.add( mid );
				}
			}
            
            //   
            // TODO: 

			result = true;
		}
		catch ( Exception ex )
		{
			ex.printStackTrace();
		}

		return result;
	}

	private ApplicationModule getModule( String packageServer )
	{
		ApplicationModule result = null;
		try
		{
			result = (ApplicationModule)Class.forName( packageServer + ".Module" ).
					 newInstance();
			result.init( con );
		}
		catch ( Exception ex )
		{
			ex.printStackTrace();
		}
		return result;
	}

	//   
	private boolean saveContract( int cid )
	{
		StringWriter sw = new StringWriter();
		Document doc = Utils.newDoc();
		boolean result = serializeContract( doc, cid );
		if ( result )
		{
			Utils.serializeDoc( doc, sw );

			if ( currentFolder != null )
				try
				{
					String title = _cu.getContractTitle( cid, true );

					FileOutputStream fos = new FileOutputStream(
									   currentFolder.getAbsolutePath() +
									   "/" + cid + "$" + title.replace( '\\' , '_' ).replace( '/', '_' ) + ".zip" );

					ZipOutputStream zos = new ZipOutputStream( fos );
					zos.putNextEntry( new ZipEntry( "archive.xml" ) );
					zos.write( sw.toString().getBytes( "Cp1251" ) );
					zos.closeEntry();
					zos.close();

					fos.flush();
					fos.close();
				}
				catch ( Exception ex )
				{
					ex.printStackTrace();
					result = false;
				}
			else
				result = false;
		}

		return result;
	}

	//         
	public List<File> getContractsFromArchive( String fileFilter )
	{
		List<File> result = new ArrayList<File>();
		if ( currentFolder != null )
		{
			File[] list = currentFolder.listFiles();
			int size = list.length;
			for ( int i = 0; i < size; i++ )
			{
				File arch = list[ i ];
				if(	arch.isFile() &&
					( fileFilter == null ||
					  arch.getName().toLowerCase().indexOf( fileFilter.toLowerCase() ) >= 0 ) )
				{
					result.add( arch );
				}
			}
		}
		else
		{
			System.err.println( "No archive folder find!" );
		}

		return result;
	}

	//    
    public String restoreContract( String fileName )
    {
        String result = null;

        if ( currentFolder != null )
        {
			try
			{
                File zipFile = new File( currentFolder.getAbsolutePath() +
										"/" + fileName );
                if( zipFile.exists() )
                {
					FileInputStream fis = new FileInputStream( zipFile );
                    ZipUtils zu = new ZipUtils( fis );
                    byte[] xml = zu.getEntry( null );

                    if( xml != null )
                    {
						Document doc = Utils.getDocumentBuilder().parse( new ByteArrayInputStream( xml ) );
						Element tables = doc.getDocumentElement();
						NodeList childs = tables.getChildNodes();
						int size = childs.getLength();

						for ( int i = 0; i < size; i++ )
						{
							Node node = childs.item( i );
							if ( node instanceof Element )
								try
								{
									insertElementToBase( (Element)node );
								}
								catch ( Exception ex )
								{
									ex.printStackTrace();
									result += "\n    :" +
										node.getNodeName();
								}
						}
					}
                    else
					{
						result = "XML is Null!!";
					}

					fis.close();

					//   ,     
					if ( result == null && !zipFile.delete() )
					{
						result = "Archive delete error!";
					}
				}
                else
                    result =  "  :" + fileName;
			}
			catch ( Exception ex )
			{
                ex.printStackTrace();
                result = ex.getMessage();
			}
        }
        else
		{
			result = "No archive folder find!";
		}
        return result;
    }

	//   XML   
    private int insertElementToBase( Element el )
        throws SQLException
    {
		int result = 0;

        String tableName = el.getNodeName();

        if( Utils.tableExists( con, tableName ) )
        {
			StringBuffer query = new StringBuffer( "INSERT INTO " );
			query.append( tableName );

            StringBuffer columns = new StringBuffer();
            StringBuffer values = new StringBuffer();

			NamedNodeMap attrs = el.getAttributes();
			int size = attrs.getLength();

			for ( int i = 0; i < size; i++ )
			{
				Node node = attrs.item( i );
				String col = node.getNodeName();
				String value = node.getNodeValue();

				columns.append( col );
				if ( i < size - 1 )
				{
				    columns.append( "," );
				}
				
				if( (col.equals( "date1" ) || col.equals( "date2" ) ) &&
				    (value.equals( "" ) || value.equals( "0000-00-00" ) ) ) 
				{
				    values.append( "NULL" );
				}
				else
				{
    				values.append( "'" );
    				values.append( value );
    				values.append( "' " );
				}
				
				if ( i < size - 1 )
				{
				    values.append( "," );
				}
			}

            query.append( " ( " );
            query.append( columns );
            query.append( " ) " );

            query.append( " VALUES(" );
            query.append( values );
            query.append( ")" );

			con.createStatement().executeUpdate( query.toString() );

			result = Utils.lastInsertId( con );
		}

		return result;
    }

	//     
	public String restoreContractClone( String fileName )
	{
		String result = null;

		if ( currentFolder != null )
		{
			try
			{
				File zipFile = new File( currentFolder.getAbsolutePath() +
										"/" + fileName );
				if( zipFile.exists() )
				{
					FileInputStream fis = new FileInputStream( zipFile );
					ZipUtils zu = new ZipUtils( fis );
					byte[] xml = zu.getEntry( null );

					if( xml != null )
					{
						Document doc = Utils.getDocumentBuilder().parse( new ByteArrayInputStream( xml ) );
						Element tables = doc.getDocumentElement();
						NodeList childs = tables.getChildNodes();
						int size = childs.getLength();

						Set<String> kernelTables = getKernelTables();

						int cid = 0;

						for ( int i = 0; i < size; i++ )
						{
							Node node = childs.item( i );
							if ( node instanceof Element )
							{
								try
								{
									String tableName = node.getNodeName();

									if ( tableName.equals( "contract" ) )
									{
										( ( Element )node ).removeAttribute( "id" );
										cid = insertElementToBase( ( Element )node );
									}
									else if ( kernelTables.contains( node.getNodeName() )
											  && cid != 0 )
									{
										( ( Element )node ).setAttribute( "cid", String.valueOf( cid ) );
										( ( Element )node ).removeAttribute( "id" );

										insertElementToBase( ( Element )node );
									}
								}
								catch ( Exception ex )
								{
									ex.printStackTrace();
									result += "\n    :" + node.getNodeName();
								}
							}
						}
					}
					else
					{
						result = "XML is Null!!";
					}

					fis.close();
				}
				else
				{
					result = "  :" + fileName;
				}
			}
			catch ( Exception ex )
			{
				ex.printStackTrace();
				result = ex.getMessage();
			}
		}
		else
		{
			result = "No archive folder find!";
		}
		return result;
	}

	//  ,     
	private Set<String> getKernelTables()
	{
		Set<String> result = new HashSet<String>();
		result.add( "contract_account" );
		result.add( "contract_balance" );
		result.add( "contract_payment" );
		result.add( "contract_charge" );
		result.add( "contract_parameter_type_1" );
		result.add( "contract_parameter_type_2" );
		result.add( "contract_parameter_type_3" );
		result.add( "contract_parameter_type_4" );
		result.add( "contract_parameter_type_5" );
		result.add( "contract_parameter_type_6" );
		result.add( "contract_parameter_type_7" );
		result.add( "contract_tariff" );
		result.add( "contract_service" );
		return result;
	}


	public List<String> getFolders()
	{
		List<String> result = new ArrayList<String>();
		if( archiveFolder != null )
		{
            File[] files = archiveFolder.listFiles();
			result.add( "/" );
			for( int i = 0; i < files.length; i++ )
			{
				if( files[i].isDirectory() )
				{
					result.add( files[ i ].getName() );
				}
			}
		}
		return result;
	}

	public String getArchiveContent( String fileName )
	{
		String result = "";

		if ( currentFolder != null )
		{
			try
			{
				File zipFile = new File( currentFolder.getAbsolutePath() +
										 "/" + fileName );
				if ( zipFile.exists() )
				{
					FileInputStream fis = new FileInputStream( zipFile );
					ZipUtils zu = new ZipUtils( fis );
					byte[] xml = zu.getEntry( null );

					if ( xml != null )
					{
						result = new String( xml, "Cp1251" );
					}
				}
			}
			catch ( Exception ex )
			{
				ex.printStackTrace();
			}
		}

		return result;
	}


	public static void main( String[] args )
	{
		SetupData setup = new SetupData( "data.data" );
		ContractRemover cr = new ContractRemover( setup, setup.getDBConnectionFromPool(), "/" );
		List<Integer> cids = new ArrayList<Integer>();
		cids.add( new Integer( 2241 ) );
		cr.removeContracts( cids, true );
	}
	

}

class ContractTable
{
	public String keyColumn;
	public String name;

	public ContractTable( String name, String keyColumn )
	{
		this.name = name;
		this.keyColumn = keyColumn;
	}
}
